@@ -35,6 +35,7 @@
 #include "presui.h"
 #include "langui.h"
 #include "scrui.h"
+#include "bmui.h"
 
 
 #include <signal.h>
@@ -47,8 +48,22 @@
 #define GN_SHKEY 34556
 #define GN_SEGSIZE 10
 
+#define ACCESSIBILITY_GCONF_KEY "/desktop/gnome/interface/accessibility"
+
 /*#define __WAIT_DEBUG__  */
 
+
+static gboolean gn_run_with_parameters (void);
+static gboolean gn_no_multiple_instance (void);
+static gboolean gn_destroy_instance (void);
+static gboolean gn_init (void);
+static gboolean	gn_init_gconf (void);
+static gboolean gn_terminate (void);
+static gboolean gn_run_srcore (void);
+static gboolean gn_child_process_wait (void);
+static gboolean gn_check_accessibility_key (void);
+static gboolean gn_gnopernicus_wait_to_core (void);
+
 /**
  *
  * Instance of "Braille setting" structure
@@ -84,58 +99,6 @@ Speech 	  		*speech_setting = NULL;
 **/
 Magnifier 		*magnifier_setting = NULL;
 
-/**
- *
- * Test the parameters list from line command.
- *
-**/
-static gboolean gn_run_with_parameters();
-
-/**
- *
- * Test if it lunched more then one time.
- * return - TRUE no multiple execution
- *
-**/
-static gboolean gn_no_multiple_instance();
-
-/**
- *
- * Destroy shared memory and message queue.
- * return - FALSE if error;
- *
-**/
-static gboolean gn_destroy_instance();
-
-/**
- *
- * Initialize all gconf client and load all settings.
- * return - FALSE error
- *
-**/
-static gboolean gn_init();
-
-/**
- *
- * Destroy all gconf client and all internal structure
- *
-**/
-static gboolean gn_terminate();
-
-/**
- *
- * Launch in executions SRCORE
- *
-**/
-static gboolean gn_run_srcore();
-
-/**
- *
- * Wait for end of execiton of child process
- *
-**/
-static gboolean gn_child_process_wait();
-
 /**
  *
  * If the settings loaded with default values?
@@ -157,6 +120,7 @@ static gint 	gn_semid;
 **/
 static gint	gn_srcore_pid;
 extern gint 	bm_pid;
+
 /**
  *
  * srcore_status - exit status of srcore process
@@ -164,12 +128,128 @@ extern gint 	bm_pid;
 **/
 static gint	gn_srcore_status;
 
-static gboolean gn_defaultmode = FALSE;
+extern gboolean init_finish;
+
+static gchar  **gn_argv;
+
+static gboolean gn_defaultmode    = FALSE;
+static gboolean gn_braille_enable = FALSE;
+static gboolean gn_speech_enable  = FALSE;
+static gboolean gn_magnifier_enable = FALSE;
+static gboolean gn_braille_monitor_enable = FALSE;
+static gboolean gn_braille_disable = FALSE;
+static gboolean gn_speech_disable  = FALSE;
+static gboolean gn_magnifier_disable = FALSE;
+static gboolean gn_braille_monitor_disable = FALSE;
+
+static gchar   *gn_braille_device = NULL;
+static gint     gn_braille_port = -1;
 
 struct poptOption poptopt[] = 
     {		
-	{"default", 	'd', POPT_ARG_NONE, &gn_defaultmode, 'd', "Lunch in execution with default settings", NULL},
-	{NULL, 		0,0, NULL, 0}
+	{
+	 "default", 	
+	 'd', 	
+	 POPT_ARG_NONE, 
+	 &gn_defaultmode, 	
+	 0, 
+	 "Launch in execution with default settings", 
+	 NULL
+	},
+	{
+	 "enable-braille",	
+	 'b', 	
+	 POPT_ARG_NONE, 
+	 &gn_braille_enable,		
+	 0, 
+	 "enable braille service", 
+	 NULL
+	},
+	{
+	 "disable-braille",	
+	 'r', 	
+	 POPT_ARG_NONE, 
+	 &gn_braille_disable,		
+	 0, 
+	 "enable braille service", 
+	 NULL
+	},
+	{
+	 "enable-speech",	
+	 's', 	
+	 POPT_ARG_NONE, 
+	 &gn_speech_enable,		
+	 0, 
+	 "enable speech service", 
+	 NULL
+	},
+	{
+	 "disable-speech",	
+	 'e', 	
+	 POPT_ARG_NONE, 
+	 &gn_speech_disable,		
+	 0, 
+	 "disable speech service", 
+	 NULL
+	},
+	{
+	 "enable-magnifier",	
+	 'm', 	
+	 POPT_ARG_NONE, 
+	 &gn_magnifier_enable,		
+	 0, 
+	 "enable magnifier service",  
+	 NULL
+	},
+	{
+	 "disable-magnifier",	
+	 'a', 	
+	 POPT_ARG_NONE, 
+	 &gn_magnifier_disable,		
+	 0, 
+	 "disable magnifier service",  
+	 NULL
+	},
+
+	{
+	 "enable-braille-monitor", 
+	 'o', 
+	 POPT_ARG_NONE, 
+	 &gn_braille_monitor_enable, 	
+	 0, 
+	 "enable braille monitor service", 
+	 NULL
+	},
+	{
+	 "disable-braille-monitor", 
+	 'n', 
+	 POPT_ARG_NONE, 
+	 &gn_braille_monitor_disable, 	
+	 0, 
+	 "disable braille monitor service", 
+	 NULL
+	},
+	{
+	 "braille-port", 
+	 'p',   
+	 POPT_ARG_INT,  
+	 &gn_braille_port, 	
+	 0, 
+	 "Serial port (ttyS)", 
+	 "ttyS[1..4]"
+	},
+	{
+	 "braille-device", 
+	 'e', 
+	 POPT_ARG_STRING, 
+	 &gn_braille_device, 	
+	 0, 
+	 "Braille Device", 
+	 "DEVICE NAME"
+	},
+	{
+	 NULL, 0,     0, NULL, 0
+	}
     };
 
 gint 
@@ -191,41 +271,57 @@ main (gint argc,gchar *argv[])
 			GNOME_PARAM_APP_DATADIR, DATADIR,
 			NULL);
     sru_log_init ();
+
+    if (!gn_init_gconf ()) 
+    {
+        sru_warning (_("Gconf initialization failed."));
+	srcore_exit_all (TRUE);
+        return EXIT_FAILURE;
+    }
                         
     if (!gn_run_with_parameters ())
     {
-	sru_warning (_("gnopi:Invalid parameters"));
+	sru_warning (_("Invalid parameters."));
 	return EXIT_FAILURE;
     }
 	
     if (!gn_no_multiple_instance ())
     {
-        sru_message (_("gnopi : Multiple instances are NOT ALLOWED.\n This instance will end."));
+        sru_message (_("Multiple instances are NOT ALLOWED. This instance will end."));
         return EXIT_FAILURE;
     }
 
     if (!gtk_init_check (&argc, &argv))
     {
-        sru_warning (_("gnopi:GTK initialization."));
+        sru_warning (_("GTK initialization failed."));
         return EXIT_FAILURE;
     }
 	
     if (!gn_run_srcore ())
     {
-        sru_warning (_("gnopi:Can not create a new process"));
+        sru_warning (_("Can not create a new process."));
         return EXIT_FAILURE;
     }
-	        
-    if (!gn_init ()) 
+    
+    gn_gnopernicus_wait_to_core ();
+	            
+    if (!gn_check_accessibility_key ())
     {
-        sru_warning (_("gnopi:Gconf initialization error"));
+	sru_warning (_("Exit gnopernicus."));
+	srcore_exit_all (TRUE);
+	return FALSE;
+    }
+
+    if (!gn_init ())
+    {
+	sru_warning (_("Gnopernicus initialization failed."));
 	srcore_exit_all (TRUE);
         return EXIT_FAILURE;
     }
 	
     if (!gn_load_gnopi ())
     {
-        sru_warning (_("gnopi:Can not load .glade2 file"));
+        sru_warning (_("Can not load .glade2 file"));
         return EXIT_FAILURE;
     }
 	
@@ -255,9 +351,214 @@ main (gint argc,gchar *argv[])
 gboolean 
 gn_run_with_parameters (void)
 {    
+    gint     index   = 0;
+    gboolean rv      = TRUE;
+    
     if (gn_defaultmode)
 	gn_loaddefault = TRUE;
+	
+#define COUNT_OF_PARAMETERS_FOR_FORK 8
+    gn_argv = (gchar**) g_new0 (gchar*, COUNT_OF_PARAMETERS_FOR_FORK);
+#undef  COUNT_OF_PARAMETERS_FOR_FORK
+
+    gn_argv[index] = NULL;
     
+    index++;
+
+    if (gn_magnifier_disable)
+    {
+	gn_argv[index] = g_strdup ("--disable-magnifier");
+	srcore_magnifier_status_set (FALSE);
+	index++;
+    }
+    else
+    if (gn_magnifier_enable)
+    {
+    	gn_argv[index] = g_strdup ("--enable-magnifier");
+	srcore_magnifier_status_set (TRUE);
+	index++;
+    }
+    
+    if (gn_speech_disable)
+    {
+	gn_argv[index] = g_strdup ("--disable-speech");
+	srcore_speech_status_set (FALSE);
+	index++;
+    }
+    else
+    if (gn_speech_enable)
+    {
+	gn_argv[index] = g_strdup ("--enable-speech");
+	srcore_speech_status_set (TRUE);
+	index++;
+    }
+    
+    if (gn_braille_disable)
+    {
+    	gn_argv[index] = g_strdup ("--disable-braille");
+	srcore_braille_status_set (FALSE);
+	index++;
+    }
+    else
+    if (gn_braille_enable)
+    {
+	gn_argv[index] = g_strdup ("--enable-braille");
+	srcore_braille_status_set (TRUE);
+	index++;
+    }
+    
+    if (gn_braille_monitor_disable)    
+    {
+    	gn_argv[index] = g_strdup ("--disable-braille-monitor");
+	srcore_braille_monitor_status_set (FALSE);
+	index++;
+    }
+    else
+    if (gn_braille_monitor_enable)    
+    {
+    	gn_argv[index] = g_strdup ("--enable-braille-monitor");
+	srcore_braille_monitor_status_set (TRUE);
+	index++;
+    }
+
+    if (gn_braille_port > -1)
+    {
+    	gn_argv[index] = g_strdup_printf ("--braille-port=%d", gn_braille_port);
+	brlconf_port_no_set (gn_braille_port);
+	index++;
+    }
+    
+    if (gn_braille_device)
+    {
+    	gn_argv[index] = g_strdup_printf ("--braille-device=%s", gn_braille_device);
+	brlconf_device_set (gn_braille_device);
+	index++;
+    }
+
+    gn_argv[index] = NULL;
+
+    return rv;
+}
+
+#define COUNT_OF_TIME_OUT_CALLS 	7
+static gboolean 
+gn_gnopernicus_syncronize (gpointer data)
+{
+    static int n = 0;
+
+    if (n < COUNT_OF_TIME_OUT_CALLS && 
+	init_finish == FALSE)
+    {
+	n++;
+	return TRUE;
+    }
+
+    gtk_main_quit ();
+    
+    return FALSE;
+}
+
+gboolean
+gn_gnopernicus_wait_to_core 	(void)
+{    
+    
+#define LOOP_EXIT_TIME_OUT_INTERVAL 300
+    g_timeout_add (LOOP_EXIT_TIME_OUT_INTERVAL, gn_gnopernicus_syncronize, NULL);
+    
+    gtk_main ();
+#undef  LOOP_EXIT_TIME_OUT_INTERVAL    
+
+    return init_finish; 
+}
+
+
+/**
+ * gn_check_accessibility_key
+ *
+ * Check if the accessibility gconf key is turned on. Turned on off the user want.
+ *
+ * return: FALSE if the gnopernicus needed to restart.
+**/
+gboolean
+gn_check_accessibility_key (void)
+{
+    GError 	*error  = NULL;
+    GtkWidget 	*dialog = NULL;
+    GladeXML	*xml    = NULL;
+    gint         response_id;
+    gboolean     accessibility_on;
+    
+    /* check if accessibility flag is TRUE */
+    accessibility_on = 	gnopiconf_get_bool (ACCESSIBILITY_GCONF_KEY, 
+					    &error);
+        
+    if (error)
+    {
+	sru_warning (_(error->message));
+	g_error_free (error);
+	return TRUE;
+    }
+        
+    if (accessibility_on)
+	return TRUE;
+
+    xml 	= gn_load_interface ("gnopi.glade2", "dl_enable");	
+    dialog 	= glade_xml_get_widget (xml, "dl_enable");
+    
+    g_object_unref (G_OBJECT (xml));
+    
+    response_id = gtk_dialog_run (GTK_DIALOG (dialog));
+
+/*    gtk_widget_destroy (dialog);*/ /* FIXME: if uncomment this it crash the application */
+    gtk_widget_hide (dialog);
+
+    if (response_id == GTK_RESPONSE_NO)
+	return TRUE;
+	
+    if (response_id == GTK_RESPONSE_YES)
+    {
+	gnopiconf_set_bool (TRUE, ACCESSIBILITY_GCONF_KEY);
+	
+	xml 	= gn_load_interface ("gnopi.glade2", "dl_logout");	
+	dialog 	= glade_xml_get_widget (xml, "dl_logout");
+	
+	g_object_unref (G_OBJECT (xml));
+	
+	response_id = gtk_dialog_run (GTK_DIALOG (dialog));
+	
+	gtk_widget_hide (dialog);
+
+	if (response_id == GTK_RESPONSE_NO)
+	    return TRUE;
+	if (response_id == GTK_RESPONSE_YES)
+	{
+	    GnomeClient *client = NULL;
+	    if (!(client = gnome_master_client ())) 
+		gtk_main_quit ();
+
+	    gnome_client_request_save (client, GNOME_SAVE_GLOBAL, TRUE, 
+					GNOME_INTERACT_ANY, FALSE, TRUE);
+	    return TRUE;
+	}
+	
+    }
+    
+    return TRUE;
+}
+
+gboolean
+gn_init_gconf (void)
+{
+    sru_return_val_if_fail (gnopiconf_gconf_client_init (), FALSE);
+    sru_return_val_if_fail (brlconf_gconf_client_init (), FALSE);
+    sru_return_val_if_fail (kbconf_gconf_client_init (), FALSE);
+    sru_return_val_if_fail (spconf_gconf_client_init (), FALSE);
+    sru_return_val_if_fail (magconf_gconf_client_init (), FALSE);
+    sru_return_val_if_fail (srcore_gconf_client_init (), FALSE);
+    sru_return_val_if_fail (cmdconf_gconf_client_init (), FALSE);
+    sru_return_val_if_fail (presconf_gconf_init (), FALSE);
+    sru_return_val_if_fail (bmconf_gconf_client_init (), FALSE);
+
     return TRUE;
 }
 
@@ -269,22 +570,13 @@ gn_run_with_parameters (void)
 **/
 gboolean 
 gn_init (void)
-{
-    sru_return_val_if_fail (gnopiconf_gconf_client_init(), FALSE);
-    sru_return_val_if_fail (brlconf_gconf_client_init (), FALSE);
-    sru_return_val_if_fail (kbconf_gconf_client_init (), FALSE);
-    sru_return_val_if_fail (spconf_gconf_client_init (), FALSE);
-    sru_return_val_if_fail (magconf_gconf_client_init (), FALSE);
-    sru_return_val_if_fail (srcore_gconf_client_init (), FALSE);
-    sru_return_val_if_fail (cmdconf_gconf_client_init (), FALSE);
-    sru_return_val_if_fail (presconf_gconf_init (), FALSE);
-        
+{        
     braille_setting   = brlconf_setting_init (TRUE);
     keyboard_setting  = kbconf_setting_init (TRUE);
     speech_setting    = spconf_setting_init (TRUE);
     general_setting   = srcore_general_setting_init (TRUE);
-        
-    if (general_setting->braille_monitor) 
+
+    if (general_setting->braille_monitor)
     {
 	genui_run_brlmonitor ();
 	sleep (1);
@@ -351,12 +643,15 @@ gn_run_srcore (void)
         g_file_test ("../srcore/srcore", G_FILE_TEST_IS_EXECUTABLE) &&
 	g_file_test ("../srcore/srcore", G_FILE_TEST_IS_REGULAR))
     {
-	gchar *args[] = {"../srcore/srcore",NULL};
-	if (!g_spawn_async ( ".", args , NULL, 
+	gn_argv[0] = g_strdup ("../srcore/srcore");
+	
+	if (!g_spawn_async ( ".", gn_argv, NULL, 
 			    G_SPAWN_DO_NOT_REAP_CHILD,
 			    NULL, NULL, &gn_srcore_pid, NULL))
 	{
-	    sru_message (_("No srcore binary file found [errno: %i]."),errno);
+	    sru_message ("No \"%s\" binary file found [errno: %i].",
+			"srcore",
+			errno);
 	    return FALSE;
 	}
     }
@@ -365,27 +660,34 @@ gn_run_srcore (void)
     	g_file_test ("./srcore", G_FILE_TEST_IS_EXECUTABLE) &&
 	g_file_test ("./srcore", G_FILE_TEST_IS_REGULAR))
     {
-	gchar *args[] = {"./srcore",NULL};
-	if (!g_spawn_async ( ".", args , NULL, 
+	gn_argv[0] = g_strdup ("./srcore");
+	
+	if (!g_spawn_async ( ".", gn_argv , NULL, 
 			G_SPAWN_DO_NOT_REAP_CHILD,
 			NULL, NULL, &gn_srcore_pid, NULL))
 	{
-	    sru_message (_("No srcore binary file found [errno: :%i]"),errno);
+	    sru_message ("No \"%s\" binary file found [errno: %i].",
+			"srcore",
+			errno);
 	    return FALSE;
 	}
     }
     else
     {
-	gchar *args[] = {"srcore",NULL};
-	if (!g_spawn_async ( ".", args, NULL, 
+	gn_argv[0] = g_strdup ("srcore");
+	if (!g_spawn_async ( ".", gn_argv, NULL, 
 			    G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
 			    NULL, NULL, &gn_srcore_pid, NULL))
 	{
-	    sru_message (_("No srcore binary file found [errno: :%i]"),errno);
+	    sru_message ("No \"%s\" binary file found [errno: %i].",
+			"srcore",
+			errno);
 	    return FALSE;
 	}
     }
 			
+    g_strfreev (gn_argv);
+    
     return TRUE;
 }
 
@@ -420,7 +722,7 @@ gn_no_multiple_instance (void)
 		
 		if (pd == 0)
     		{
-		    sru_error (_("No srcore in execution"));
+		    sru_error ("No srcore in execution.");
 		    return FALSE;
 		}
 
