@@ -48,6 +48,8 @@ gboolean	gnopiconf_set_bool (gboolean val,
 gboolean	gnopiconf_set_bool_in_section (gboolean    val, 
 					       const gchar *section, 
 					       const gchar *key);
+gboolean	gnopiconf_get_bool (const gchar* key,
+				    GError **error);
 gboolean	gnopiconf_set_list (GSList *list, 
 				    GConfValueType list_type, 
 				    const gchar *key);
