@@ -35,6 +35,11 @@ static GtkWidget *cb_pres_add_role;
 static GtkWidget *cb_pres_add_device;
 static GtkWidget *cb_pres_add_event;
 static GtkWidget *bt_pres_add_paste;
+static GtkWidget *bt_pres_chunk_remove;
+static GtkWidget *bt_pres_chunk_up;
+static GtkWidget *bt_pres_chunk_down;
+static GtkWidget *bt_presentation_modify;
+static GtkWidget *bt_presentation_remove;
 static GtkWidget *et_pres_add_chunks;
 static GtkTreeIter current_iter;
 static gboolean	 new_item_in_list;
@@ -45,6 +50,118 @@ static gboolean  pres_add_changed;
 static gchar  *copy_paste = NULL; 
 extern GSList *pres_list;
 static GSList *current_item = NULL;
+
+const gchar *pres_device_list[] =
+{
+    "braille",
+    "speech",
+    "magnifier"
+};
+
+const gchar *pres_event_list[] =
+{
+    "generic",
+    "present",
+    "focus",
+    "object:text-caret-moved",
+    "object:text-changed:insert",
+    "object:text-changed:delete",
+    "object:text-selection-changed",
+    "object:property-change:accessible-value",
+    "object:state-changed:checked",
+    "object:state-changed:selected",
+    "object:state-changed:expanded",
+    "object:state-changed:active",
+    "object:state-changed:defunct",
+    "object:property-changed:accessible-content",
+    "tooltip:show",
+    "tooltip:hide",
+    "window:create",
+    "window:destroy",
+    "window:minimize",
+    "window:titlelize",
+    "window:restore",
+    "window:maximize",
+    "window:switch",
+};
+
+
+const gchar *pres_role_list[] =
+    {
+	"unknown",	
+	"invalid",	
+        "alert", 	
+	"canvas",	
+        "check-box",	
+	"check-menu-item",
+        "column-header",
+	"combo-box",	
+        "desktop-icon",
+	"desktop-frame",
+        "dialog",	
+	"directory-pane",
+        "file-chooser",
+	"filler",	
+        "frame",	
+        "glass-pane",	
+	"HTML-container",		
+        "icon",			
+        "internal-frame",		
+	"label",			
+        "layered-pane",		
+	"link",			
+	"list",			
+        "list-item",			
+	"menu",			
+        "menu-bar",			
+	"menu-item",			
+        "option-pane",			
+        "page-tab",			
+	"page-tab-list",		
+        "panel",			
+        "password-text",		
+	"popup-menu",			
+        "progress-bar",		
+	"push-button",			
+        "radio-button",		
+	"radio-menu-item",		
+        "root-pane",			
+	"row-header",			
+        "scroll-bar",			
+	"scroll-pane",			
+        "separator",			
+	"slider",			
+        "split-pane",			
+	"status-bar",			
+        "table",			
+	"table-cell",			
+        "table-column-header",		
+	"table-row-header",		
+        "text",			
+	"toggle-button",		
+        "tool-bar",			
+	"tool-tip",			
+        "tree",			
+        "tree-item",			
+	"tree-table",			
+	"viewport",			
+	"window",			
+	"accelerator-label",		
+        "animation",			
+	"arrow",			
+        "calendar",			
+	"date-editor",			
+        "dial",			
+	"drawing-area",		
+        "font-chooser",		
+	"image",			
+        "spin-button",			
+	"terminal",			
+        "extended",			
+	"table-line",			
+        "table-columns-header",	
+	"title-bar"
+    };
 enum
 {
     PRESENTATION_DEVICE,
@@ -90,7 +207,8 @@ enum
 {
     PRES_TAG_IDLE = 0,
     PRES_TAG_START,
-    PRES_TAG_BRACKET
+    PRES_TAG_BRACKET,
+    PRES_TAG_TEXT
 };
 
 static GSList*
@@ -104,17 +222,27 @@ presui_split_chunk (const gchar *chunk)
     sru_return_val_if_fail (chunk, rv);
     
     iter = (gchar*)chunk;
-    
+
     while (iter && *iter != '\0')
     {
 	switch (tag_state)
 	{
 	    case PRES_TAG_IDLE:
+	    {
+		start = iter;
+		if (*iter == '%')
+		    tag_state = PRES_TAG_START;
+		else
+		    tag_state = PRES_TAG_TEXT;
+	    }
+	    break;
+	    case PRES_TAG_TEXT:
 	    {
 		if (*iter == '%')
 		{
-		    tag_state = PRES_TAG_START;
-		    start = iter;
+		    rv = g_slist_append (rv, g_strndup (start, iter - start));
+		    tag_state = PRES_TAG_IDLE;
+		    iter--;
 		}
 	    }
 	    break;
@@ -143,6 +271,12 @@ presui_split_chunk (const gchar *chunk)
 	iter++;
     }
     
+    if (*iter 	  == '\0' &&
+	tag_state == PRES_TAG_TEXT)
+    {
+	rv = g_slist_append (rv, g_strndup (start, iter - start));
+    }
+    
     return rv;
 }
 
@@ -176,7 +310,7 @@ presui_get_chunk (void)
 	    str = tmp;
 	}
 	g_free (key);
-	valid = gtk_tree_model_iter_next ( model, &iter);
+	valid = gtk_tree_model_iter_next (model, &iter);
     }
 
     return str;
@@ -221,10 +355,12 @@ presui_pres_update_list (void)
 				    	    ));
 	chunk = presui_get_chunk ();
 	
+	if (!chunk)
+	    chunk = "";
+	
 	if (new_item_in_list)
 	{
-	    if (!presconf_item_exist (device, role, event) &&
-		strlen (chunk) > 0)
+	    if (!presconf_item_exist (device, role, event))
 	    {
 		presconf_append_item_to_list (	device,
 			    			role,
@@ -238,7 +374,10 @@ presui_pres_update_list (void)
 		new_item_in_list = FALSE;
 	    }
 	    else
+	    {
+		gn_show_message (_("Invalid presentation item!"));
 		set = FALSE;
+	    }
 	}
 	else
 	{
@@ -251,7 +390,6 @@ presui_pres_update_list (void)
 		current_item = NULL;
 	    }
 	}
-	
 	if (set)
 	{
 	    gtk_list_store_set (GTK_LIST_STORE (model), 
@@ -264,7 +402,6 @@ presui_pres_update_list (void)
 	}
 	g_free (chunk);
     }
-
 }
 static void
 presui_pres_add_copy_clicked  (GtkWidget 	*widget,
@@ -508,9 +645,20 @@ presui_pres_add_et_activate (GtkWidget 	*widget,
 			    gpointer	user_data)
 {
     presui_presentation_add_apply_sensitivity (TRUE);
-    presui_show_chunk_in_list ( gtk_entry_get_text ( GTK_ENTRY (widget) ));
+    presui_show_chunk_in_list (gtk_entry_get_text (GTK_ENTRY (widget)));
 }
 
+static gboolean
+presui_pres_focus_out_event (GtkWidget *et_chunk,
+			    GdkEvent  *event,
+		    	    gpointer user_data)
+{
+    presui_presentation_add_apply_sensitivity (TRUE);
+    presui_show_chunk_in_list (gtk_entry_get_text (GTK_ENTRY (et_chunk)));
+    return FALSE;
+}
+
+
 static void
 presui_response_presentation_add_event (GtkDialog *dialog,
 					gint       response_id,
@@ -552,6 +700,71 @@ presui_delete_emit_response_cancel (GtkDialog *dialog,
     return TRUE; /* Do not destroy */
 }
 
+static void
+presui_fill_role_combo (void)
+{
+    GList *list = NULL;
+    gint iter;
+    
+    for (iter = 0; iter < G_N_ELEMENTS (pres_role_list); iter++)
+	list = g_list_append (list, (gpointer)pres_role_list[iter]);
+    
+    gtk_combo_set_popdown_strings (GTK_COMBO (cb_pres_add_role), list);
+    
+    for (list = g_list_first (list) ; list; list = g_list_next (list))
+	list->data = NULL;
+	
+    g_list_free (list);
+}
+
+static void
+presui_fill_event_combo (void)
+{
+    GList *list = NULL;
+    gint iter;
+    
+    for (iter = 0; iter < G_N_ELEMENTS (pres_event_list); iter++)
+	list = g_list_append (list, (gpointer)pres_event_list[iter]);
+    
+    gtk_combo_set_popdown_strings (GTK_COMBO (cb_pres_add_event), list);
+    
+    for (list = g_list_first (list) ; list; list = g_list_next (list))
+	list->data = NULL;
+	
+    g_list_free (list);
+}
+
+static void
+presui_fill_device_combo (void)
+{
+    GList *list = NULL;
+    gint iter;
+    
+    for (iter = 0; iter < G_N_ELEMENTS (pres_device_list); iter++)
+	list = g_list_append (list, (gpointer)pres_device_list[iter]);
+    
+    gtk_combo_set_popdown_strings (GTK_COMBO (cb_pres_add_device), list);
+    
+    for (list = g_list_first (list) ; list; list = g_list_next (list))
+	list->data = NULL;
+	
+    g_list_free (list);
+}
+
+static void
+presui_presentation_add_selection_changed (GtkTreeSelection *selection,
+				    	    gpointer  	user_data)
+{
+    gboolean 	sensitive;
+    
+    sensitive = gtk_tree_selection_get_selected (selection, NULL, NULL);
+    
+    gtk_widget_set_sensitive (GTK_WIDGET (bt_pres_chunk_down), sensitive);
+    gtk_widget_set_sensitive (GTK_WIDGET (bt_pres_chunk_up), sensitive);
+    gtk_widget_set_sensitive (GTK_WIDGET (bt_pres_chunk_remove), sensitive);
+}
+
+
 static void
 presui_set_handler_presentation_add (GladeXML *xml)
 {
@@ -568,11 +781,22 @@ presui_set_handler_presentation_add (GladeXML *xml)
     bt_pres_add_paste	= glade_xml_get_widget (xml, "bt_pres_add_paste");
     et_pres_add_chunks	= glade_xml_get_widget (xml, "et_pres_chunks");
 
+    bt_pres_chunk_remove= glade_xml_get_widget (xml, "bt_pres_chunk_remove");
+    bt_pres_chunk_up	= glade_xml_get_widget (xml, "bt_pres_chunk_up");
+    bt_pres_chunk_down	= glade_xml_get_widget (xml, "bt_pres_chunk_down");
+
+    gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (cb_pres_add_device)->entry), FALSE);
+    gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (cb_pres_add_role)->entry), FALSE);
+    gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (cb_pres_add_event)->entry), FALSE);
+    
     g_signal_connect (w_presentation_add, "response",
 		      G_CALLBACK (presui_response_presentation_add_event), NULL);
     g_signal_connect (w_presentation_add, "delete_event",
                       G_CALLBACK (presui_delete_emit_response_cancel), NULL);
 
+    g_signal_connect (et_pres_add_chunks, "focus_out_event",
+		      G_CALLBACK (presui_pres_focus_out_event), NULL);
+		      
     glade_xml_signal_connect (xml,"on_et_pres_chunks_activate",		
 			    GTK_SIGNAL_FUNC (presui_pres_add_et_activate));
 			    
@@ -599,6 +823,14 @@ presui_set_handler_presentation_add (GladeXML *xml)
 			    
     model = presui_create_add_model ();
     
+    gtk_tree_view_set_model (GTK_TREE_VIEW (tv_pres_add_chunk), model);
+
+    selection = gtk_tree_view_get_selection ( GTK_TREE_VIEW (tv_pres_add_chunk));        
+
+    g_signal_connect (G_OBJECT (selection), "changed",
+		      G_CALLBACK (presui_presentation_add_selection_changed), NULL);
+    
+        
     gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (tv_pres_add_chunk),
 					  GDK_BUTTON1_MASK,
 					  row_targets,
@@ -613,8 +845,7 @@ presui_set_handler_presentation_add (GladeXML *xml)
     g_signal_connect ( G_OBJECT (tv_pres_add_chunk), "drag_data_received",
 			G_CALLBACK (presui_drop_cb), NULL);
     
-    gtk_tree_view_set_model (GTK_TREE_VIEW (tv_pres_add_chunk), model);
-    
+
     g_object_unref (G_OBJECT (model));
 
     cell_renderer = gtk_cell_renderer_text_new ();
@@ -630,8 +861,12 @@ presui_set_handler_presentation_add (GladeXML *xml)
     gtk_tree_view_column_set_clickable (column, TRUE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (tv_pres_add_chunk), column);
     
-    selection = gtk_tree_view_get_selection ( GTK_TREE_VIEW (tv_pres_add_chunk));
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);    
+    presui_presentation_add_selection_changed (selection, NULL);
+    
+    presui_fill_role_combo ();
+    presui_fill_event_combo ();
+    presui_fill_device_combo ();
     
     presui_presentation_add_paste_sensitivity (FALSE);
 }
@@ -786,7 +1021,7 @@ presui_pres_add_clicked (GtkWidget 	*widget,
     new_item_in_list = TRUE;
     presui_apply_sensitivity (TRUE);
     if (!presui_load_presentation_add ())
-	sru_warning (_("We colud not load interface"));
+	sru_warning (_("We could not load interface."));
 }
 
 static void
@@ -824,16 +1059,27 @@ presui_pres_remove_clicked (GtkWidget 	*widget,
 				      event, chunk);
 	}
     }
+    else
+	gn_show_message (_("No selected presentation to remove!"));
 }
 
 static void
 presui_pres_modify_clicked (GtkWidget 	*widget,
 			gpointer	data)
 {
+    GtkTreeSelection 	*selection;
+    selection 	= gtk_tree_view_get_selection 	( GTK_TREE_VIEW (tv_presentation_order) );	    
+
+    if (!gtk_tree_selection_get_selected (selection, NULL, NULL))
+    {
+	gn_show_message (_("No selected presentation to modify!"));
+	return;
+    }
+
     new_item_in_list = FALSE;
     presui_apply_sensitivity (TRUE);
     if (!presui_load_presentation_add ())
-	sru_warning (_("We colud not load interface"));
+	sru_warning (_("We could not load interface."));
 }
 
 
@@ -842,9 +1088,47 @@ presui_row_activated_cb (GtkTreeView       *tree_view,
                 	GtkTreePath       *path,
 			GtkTreeViewColumn *column)
 {
+    GtkTreeSelection 	*selection;
+    selection 	= gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_presentation_order) );	    
+
+    if (!gtk_tree_selection_get_selected (selection, NULL, NULL))
+    {
+	gn_show_message (_("No selected presentation to modify!"));
+	return;
+    }
+
     presui_pres_modify_clicked (NULL, NULL);
 }
 
+static void
+presui_presentation_selection_changed (GtkTreeSelection *selection,
+				       gpointer  	user_data)
+{
+    gboolean 	sensitive;
+    
+    sensitive = gtk_tree_selection_get_selected (selection, NULL, NULL);
+    
+    gtk_widget_set_sensitive (GTK_WIDGET (bt_presentation_modify), sensitive);
+    gtk_widget_set_sensitive (GTK_WIDGET (bt_presentation_remove), sensitive);
+}
+
+static gboolean
+presui_search_equal_func (GtkTreeModel  *model,
+			 gint 		column,
+			 const gchar	*key,
+			 GtkTreeIter	*iter,
+			 gpointer	search_data)
+{
+    gchar *list_key = NULL;
+    
+    gtk_tree_model_get (model, 	iter,
+    			column, &list_key,
+                    	-1);
+
+    return g_strcasecmp (key, list_key) > 0; 
+}
+
+
 static void
 presui_response_event (GtkDialog *dialog,
 			gint       response_id,
@@ -901,6 +1185,9 @@ presui_set_handlers (GladeXML *xml)
 
     w_presentation 	  = glade_xml_get_widget (xml, "w_presentation");
     tv_presentation_order = glade_xml_get_widget (xml, "tv_presentation_order");
+    
+    bt_presentation_remove = glade_xml_get_widget (xml, "bt_pres_remove");
+    bt_presentation_modify = glade_xml_get_widget (xml, "bt_pres_modify");
 
     g_signal_connect (w_presentation, "response",
 		      G_CALLBACK (presui_response_event), NULL);
@@ -918,6 +1205,10 @@ presui_set_handlers (GladeXML *xml)
                 
     gtk_tree_view_set_model (GTK_TREE_VIEW (tv_presentation_order), model);
 
+    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_presentation_order));
+
+    g_signal_connect (selection, "changed", 
+		      G_CALLBACK (presui_presentation_selection_changed), NULL);
     
     gtk_tree_sortable_set_sort_column_id ( GTK_TREE_SORTABLE (model), 
 					    PRESENTATION_DEVICE, 
@@ -966,9 +1257,15 @@ presui_set_handlers (GladeXML *xml)
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (tv_presentation_order), column);
 
-    selection = gtk_tree_view_get_selection ( GTK_TREE_VIEW (tv_presentation_order));
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);    
-
+    presui_presentation_selection_changed (selection, NULL);
+    
+    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tv_presentation_order), 
+				     PRESENTATION_DEVICE);    
+    gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (tv_presentation_order),
+					presui_search_equal_func,
+					NULL, NULL);
+    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tv_presentation_order), TRUE);
 }
 
 
