@@ -20,11 +20,12 @@
  */
 
 #include <stdio.h>
+#include <sys/types.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#include <strings.h>
 
 #include <glib.h>
-#include <linc/linc.h>
 #include "libsrconf.h"
 #include "srbrl.h"
 #include "brlinp.h"
@@ -32,7 +33,8 @@
 #include <unistd.h>
 #include "../braille/libbrl/brlxmlapi.h"
 #include "srpres.h"
-
+/*#include "sockaddr.h"*/
+#include "util.h"
 
 #define MIN_UDP_PORT 		1024
 #define MAX_UDP_PORT 		30000
@@ -41,17 +43,17 @@
 #define REMOTE_CONFIG_PATH 	"config/remote"
 #define BRLMON_CONFIG_PATH 	"config/brlmon"
 
-#define MIN_BRAILLE_PORT 1
-#define MAX_BRAILLE_PORT 4
 
-
-/*#define _BRLMON_DEBUG_*/
+#undef _BRAILLE_DEBUG_
 
 gchar *src_braille_device 	  = NULL;
 gint  src_braille_port 		  = -1;
 gint  src_braille_position_sensor = -1;
 gint  src_braille_optical_sensor  = -1;
-gint  src_braille_offset = 0;
+gint  src_braille_offset 	  = 0;
+gchar *src_braille_style 	  = NULL;
+gchar *src_braille_cursor_style   = NULL;
+gchar *src_braille_translation_table = NULL;
 
 extern gboolean src_use_braille;
 extern gboolean src_use_braille_monitor;
@@ -62,17 +64,6 @@ gint 	brlmon_sd, 	/* BrlMonitor socket descriptor*/
 	remote_sd;	/* RemoteBraille socket descriptor*/
 
 
-extern struct sockaddr * 
-linc_protocol_get_sockaddr (const LINCProtocolInfo *proto,
-			    const char             *hostname,
-			    const char             *service,
-			    LincSockLen            *saddr_len);
-
-extern void
-linc_protocol_destroy_addr (const LINCProtocolInfo  *proto,
-			    int                      fd,
-			    struct sockaddr         *saddr);
-
 /* socket structures */
 #ifdef INET6
     static struct sockaddr_in6	client_addr, 
@@ -91,8 +82,6 @@ linc_protocol_destroy_addr (const LINCProtocolInfo  *proto,
     #define FL_AF_INET 	AF_INET
     #define IP_TYPE	"IPv4"
 #endif
-static LINCProtocolInfo 	*remote_proto;
-static LINCProtocolInfo 	*brlmon_proto;
 GIOChannel 			*udp_giochh;
 
 /* state flag of remote modules */
@@ -297,22 +286,11 @@ brl_remote_udp_init()
 	    brlip = g_strdup (DEFAULT_REMOTE_IP);
 	}
 	
-    remote_proto = linc_protocol_find (IP_TYPE);
-    
-    if (remote_proto == NULL || remote_proto->get_sockinfo == NULL) {
-	    g_warning ("Invalid protocol type");
-	    remotesockstate = FALSE; 
-	    free (brlip);
-	    return FALSE;
-	}
-	
-    remote_braille_addr = (struct sockaddrin *) linc_protocol_get_sockaddr (
-		remote_proto, 
+    remote_braille_addr = (struct sockaddrin *) get_sockaddr (
 		brlip,
 		g_strdup_printf ("%d",brlport), 
 		(LincSockLen*)&remote_braille_addr_len);    
 
-
     
     if ((remote_sd = socket (FL_AF_INET, SOCK_DGRAM, 0)) < 0) {
 	    g_warning ("Can not open socket");
@@ -396,17 +374,7 @@ brl_mon_udp_init()
 	    monip = g_strdup (DEFAULT_BRLMON_IP);
 	}
 	
-    brlmon_proto = linc_protocol_find (IP_TYPE);
-    
-    if (brlmon_proto == NULL || brlmon_proto->get_sockinfo == NULL) {
-	    g_warning ("Invalid protocol type");
-	    brlmonsockstate = FALSE; 
-	    free (monip);
-	    return FALSE;
-	}
-
-    remote_brlmon_addr = (struct sockaddrin *) linc_protocol_get_sockaddr (
-		brlmon_proto, 
+    remote_brlmon_addr = (struct sockaddrin *) get_sockaddr (
 		monip,
 		g_strdup_printf ("%d",monport), 
 		(LincSockLen*)&remote_brlmon_addr_len);		        
@@ -467,15 +435,6 @@ src_brlmon_terminate()
     {
 	src_brlmon_quit ();
 	close (brlmon_sd);
-/*#ifdef INET6
-	linc_protocol_destroy_addr (brlmon_proto,
-				    brlmon_sd,
-				    &(remote_brlmon_addr->sin6_addr));
-#else
-	linc_protocol_destroy_addr (brlmon_proto,
-				    brlmon_sd,
-				    ((remote_brlmon_addr->sin_addr).s_addr));
-#endif*/
 	brlmonsockstate = FALSE;
 	}
 }
@@ -485,15 +444,6 @@ brl_remote_udp_terminate()
 {
     if (remotesockstate == TRUE) {
 	close (remote_sd);
-/*#ifdef INET6
-	linc_protocol_destroy_addr (remote_proto,
-				    remote_sd,
-				    &(remote_braille_addr->sin6_addr));
-#else
-	linc_protocol_destroy_addr (remote_proto,
-				    remote_sd,
-				    &((remote_braille_addr->sin_addr).s_addr));
-#endif*/
 	remotesockstate = FALSE;
 	}
 }
@@ -581,7 +531,7 @@ src_brlmon_show (gchar *message)
     text = src_xml_format ("TEXT", NULL, message);
     sprintf (offset, "%d", src_braille_offset);
     brlout = g_strconcat (
-		"<BRLOUT language=\"us\" bstyle=\"8\" clear=\"true\">\n",
+		"<BRLOUT language=\"", src_braille_translation_table,"\" bstyle=\"", src_braille_style ,"\" clear=\"true\">\n",
 		/* main display */
 		"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\">\n",
 		text,
@@ -589,7 +539,7 @@ src_brlmon_show (gchar *message)
 		/* nothing to do on state display here*/
 		"</BRLOUT>\n", 
 		NULL);
-	
+
     src_brlmon_send  (brlout);
     g_free (brlout);						
     g_free(text);
@@ -603,8 +553,9 @@ src_braille_send (gchar *brloutput)
     if (brloutput)
     {
 	brl_xml_output(brloutput, strlen(brloutput));
-/*	src_brlmon_send(brloutput);*/
-	/* fprintf (stderr, "\n%s", brloutput); */
+#ifdef _BRAILLE_DEBUG_
+	fprintf (stderr, "\n%s", brloutput); 
+#endif
     }
 }
 
@@ -618,7 +569,7 @@ src_braille_show (gchar *message)
     text = src_xml_format ("TEXT", NULL, message);
     sprintf (offset, "%d", src_braille_offset);	
     brlout = g_strconcat (
-		"<BRLOUT language=\"us\" bstyle=\"8\" clear=\"true\">\n",
+		"<BRLOUT language=\"",src_braille_translation_table,"\" bstyle=\"", src_braille_style,"\" clear=\"true\">\n",
 		/* main display */
 		"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\">\n",
 		text,
@@ -682,11 +633,13 @@ src_braille_init ()
 		&src_braille_device, default_device);
     }
 	
-    if (MIN_BRAILLE_PORT > src_braille_port || MAX_BRAILLE_PORT < src_braille_port) /* just for sure */
+    if (MIN_BRAILLE_PORT > src_braille_port || 
+	MAX_BRAILLE_PORT < src_braille_port) /* just for sure */
     {
 	gint default_port = DEFAULT_BRAILLE_PORT_NO;
 	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_PORT_NO, CFGT_INT, 
 		    &src_braille_port, &default_port);
+		    
 	if (MIN_BRAILLE_PORT > src_braille_port || MAX_BRAILLE_PORT < src_braille_port) 
 	{
 	    src_braille_port = DEFAULT_BRAILLE_PORT_NO;
